import Data from "@/bluetooth/Data.js";

//弹出提示
const msg = (title, icon = 'none', mask = false, duration = 2000) => {
	//统一提示方便全局修改
	if (Boolean(title) === false) {
		return;
	}
	uni.showToast({
		title,
		duration,
		mask,
		icon
	});
	setTimeout(function() {
		uni.hideToast();
	}, duration);
}

//确认框
function confirm(content, success, title = '') {
	var language = uni.getStorageSync('language');
	uni.showModal({
		title: title,
		content: content,
		success: function(res) {
			if (res.confirm) {
				success();
			} else if (res.cancel) {
				//console.log('用户点击取消');
			}
		},
		cancelText: language.j,
		confirmText: language.k
	});
}

//处理异常结果
function ErrorCode(obj, source) {
	var language = uni.getStorageSync('language');
	if (obj) {
		var code = obj.code;
		var message = obj.message;
		switch (code) {
			case -1:
				msg(language.b1);
				break;
			case 0:
				break;
			case 10000:
				msg(language.b2);
				break;
			case 10001:
				msg(language.b3);
				break;
			case 10002:
				msg(language.b4);
				break;
			case 10003:
				msg(language.b5);
				break;
			case 10004:
				msg(language.b6);
				break;
			case 10005:
				msg(language.b7);
				break;
			case 10006:
				msg(language.b8);
				break;
			case 10007:
				msg(language.b9);
				break;
			case 10009:
				msg(language.b10);
				break;
			case 10012:
				msg(language.b11);
				break;
			default:
				msg(language.b12 + code);
				console.log('未处理的错误', obj);
				break;
		}
	}
}

var bleCallBack = {
	CharacteristicValueChange: function(res) {
		console.log('未设置数据修改回调函数！');
	},
	ConnectionStateChange: function(res) {
		console.log('未设置状态修改回调函数！');
	},
	notifyBLE: function() {
		console.log('未设置通知功能回调函数！');
	}
};

//初始化蓝牙
function Init(callback) {
	plus.bluetooth.openBluetoothAdapter({
		success: function(res) {
			if (res.code == 0) {
				//开启监听连接状态
				plus.bluetooth.onBLEConnectionStateChange(function(event) {
					if (event) {
						var deviceId = event.deviceId;
						var connected = event.connected;
						getApp().globalData.state = connected;
						if (connected) {
							getApp().globalData.deviceId = deviceId;
							uni.setStorageSync('deviceId', deviceId);
							plus.bluetooth.setBLEMTU({
								deviceId:deviceId,
								mtu:247
							})
						} else {
							getApp().globalData.deviceId = '';
							// for (var i = 0; i < getApp().globalData.data.length; i++) {
							// 	getApp().globalData.data[i].value = 0;
							// }
						}
						bleCallBack.ConnectionStateChange(event);
					}
				});
				//监听低功耗蓝牙设备的特征值变化事件
				plus.bluetooth.onBLECharacteristicValueChange(function(event) {
					console.log(event);
					if (event) {
						var deviceId = event.deviceId;
						var serviceId = event.serviceId;
						var characteristicId = event.characteristicId;
						var value = event.value;
						var appDeviceId = getApp().globalData.deviceId;
						var appServiceId = getApp().globalData.serviceUUID;
						var appCharacteristicId = getApp().globalData.characteristicsUUID;
						// var start = getApp().globalData.start;
						// var len = getApp().globalData.len;
						//start != '' && len != '' && 
						if (deviceId == appDeviceId && serviceId == appServiceId && characteristicId == appCharacteristicId) {
							var Hex = Data.bufferToHex(value);
							var crc16Result = Data.checkCRC16(Hex);
							console.log("接受到的数据：" + Hex + ';CRC16结果：' + crc16Result);
							if (crc16Result) {
								//Data.solutionData(Hex, start, len);
								bleCallBack.CharacteristicValueChange(Hex);
							}
						}
						// getApp().globalData.start = '';
						// getApp().globalData.len = '';
					}
				});

				if (callback) {
					callback();
				}
			} else {
				ErrorCode(res, '蓝牙初始化success');
			}
		},
		fail: function(res) {
			ErrorCode(res, '蓝牙初始化fail');
		}
	});
}

//连接蓝牙
function Connect(deviceId, callback) {
	var id = getApp().globalData.deviceId;
	var state = getApp().globalData.state;
	var language = uni.getStorageSync('language');
	if (state && id != '') {
		Close(id);
	}
	uni.showLoading({
		title: language.b13,
		mask: true
	});
	getApp().globalData.isNotify = false;
	plus.bluetooth.createBLEConnection({
		deviceId: deviceId,
		success: function(res) {
			if (res.code == 0) {
				var serviceId = getApp().globalData.serviceUUID;
				var characteristicId = getApp().globalData.characteristicsUUID;
				//启用低功耗蓝牙设备特征值变化时的notify功能，订阅特征值
				setTimeout(function() {
					plus.bluetooth.notifyBLECharacteristicValueChange({
						deviceId: deviceId,
						serviceId: serviceId,
						characteristicId: characteristicId,
						success: function(e) {
							if (e.code == 0) {
								console.log('启用低功耗蓝牙设备特征值变化时的notify功能，订阅特征值:' + e.code + ':' + e.message);
								setTimeout(function() {
									getApp().globalData.isNotify = true;
									uni.hideLoading();
									msg(language.b14);
									if (callback) {
										callback();
									}
									if (bleCallBack.notifyBLE) {
										bleCallBack.notifyBLE();
									}
								}, 1500);
							} else {
								ErrorCode(e, '启用notify功能success');
							}
						},
						fail: function(e) {
							ErrorCode(e, '启用notify功能fail');
						}
					});
				}, 1500);
			} else {
				ErrorCode(res, '连接蓝牙success');
			}
		},
		fail: function(res) {
			if (res.code == 10000) {
				Init(function() {
					Connect(deviceId);
				});
			} else {
				ErrorCode(res, '连接蓝牙fail');
			}
		}
	});
}

//断开连接
function Close(deviceId) {
	plus.bluetooth.closeBLEConnection({
		deviceId,
		success(res) {
			if (res.code == 0) {

			} else {
				ErrorCode(res, '断开连接success');
			}
		},
		fail(res) {
			ErrorCode(res, '断开连接fail');
		}
	})
}

//搜索蓝牙
function SearchBluetooth(callback) {
	var language = uni.getStorageSync('language');
	plus.bluetooth.startBluetoothDevicesDiscovery({
		allowDuplicatesKey: false, //是否重复上报
		interval: 0, //上报设备的间隔。0 表示找到新设备立即上报，其他数值根据传入的间隔上报。
		success(res) {
			//console.log(res);
			plus.bluetooth.onBluetoothDeviceFound(callback);
		},
		fail() {
			msg(language.b15);
		}
	})
}

//停止搜索蓝牙
function StopSeachBluetooth() {
	plus.bluetooth.stopBluetoothDevicesDiscovery({
		success(res) {
			console.log('蓝牙已停止搜索', res);
		}
	});
}

function BluetoothState() {
	plus.bluetooth.onBluetoothAdapterStateChange(function(res) {
		if (res) {
			if (!res.discovering) {
				msg('不支持蓝牙设备！');
			}
			if (res.available) {
				msg('正在搜索蓝牙！');
			}
		}
	});
}

//写入二进制
function WriteBase(deviceId, value, callback) {

	var serviceId = getApp().globalData.serviceUUID;
	var characteristicId = getApp().globalData.writeUUID;

	plus.bluetooth.writeBLECharacteristicValue({
		// 这里的 deviceId 需要在 getBluetoothDevices 或 onBluetoothDeviceFound 接口中获取
		deviceId,
		// 这里的 serviceId 需要在 getBLEDeviceServices 接口中获取
		serviceId,
		// 这里的 characteristicId 需要在 getBLEDeviceCharacteristics 接口中获取
		characteristicId,
		// 这里的value是ArrayBuffer类型
		value: value,
		success(res) {
			if (res.code == 0) {
				if (callback) {
					callback(res);
				}
			} else {
				ErrorCode(res, '写入二进制success');
			}
		},
		fail(res) {
			ErrorCode(res, '写入二进制fail');
		}
	})
}

//写入读取命令03
function writeToRead(start, len, callback, isAuto = true) {
	var deviceId = getApp().globalData.deviceId;
	var state = getApp().globalData.state;
	var language = uni.getStorageSync('language');
	if (deviceId != '' && state) {
		var command = Data.readCommand(start, len);
		//console.log(command);
		WriteBase(deviceId, command, function(res) {
			if (callback) {
				callback(res);
			}
			// getApp().globalData.start = start;
			// getApp().globalData.len = len;
		});
	} else {
		if (!isAuto) {
			msg(language.b16);
		}
	}
}

//写入写命令06
function writeToWrite(start, value, callback) {
	var deviceId = getApp().globalData.deviceId;
	var state = getApp().globalData.state;
	var language = uni.getStorageSync('language');
	if (deviceId != '' && state) {
		var command = Data.writeCommand(start, value);
		//console.log(command);
		WriteBase(deviceId, command, function(res) {
			if (callback) {
				callback(res);
			}
		});
	} else {
		msg(language.b16);
	}
}

export default {
	msg: msg,
	confirm: confirm,
	ErrorCode: ErrorCode,

	bleCallBack: bleCallBack,

	Init: Init,
	Connect: Connect,
	Close: Close,

	SearchBluetooth: SearchBluetooth,
	BluetoothState: BluetoothState,
	StopSeachBluetooth: StopSeachBluetooth,

	WriteBase: WriteBase,
	writeToRead: writeToRead,
	writeToWrite: writeToWrite
}
