import CRC from "@/bluetooth/CRC16.js";

const top = '01';
const read = '03';
const write = '06';

//组装读取命令
function readCommand(start, len) {
	var d = top + read + start + len;
	var crc16 = CRC.ToModbusCRC16(d);
	d = d + crc16;

	var typedArray = new Uint8Array(d.match(/[\da-f]{2}/gi).map(function(h) {
		return parseInt(h, 16)
	}));
	//console.log(typedArray);
	return typedArray.buffer;
}

//组装写入命令
function writeCommand(start, value) {
	var d = top + write + start + value;
	var crc16 = CRC.ToModbusCRC16(d);
	d = d + crc16;

	var typedArray = new Uint8Array(d.match(/[\da-f]{2}/gi).map(function(h) {
		return parseInt(h, 16)
	}));
	//console.log(typedArray);
	return typedArray.buffer;
}

//arrayBuffer转16进制字符串
function ab2hex(buffer) {
	const hexArr = Array.prototype.map.call(
		new Uint8Array(buffer),
		function(bit) {
			return ('00' + bit.toString(16)).slice(-2)
		}
	)
	return hexArr.join('');
}

//验证CRC16
function checkCRC16(str) {
	if (!str) {
		return false;
	}
	str = str.toUpperCase();
	if (str.length < 6) {
		return false;
	}
	var d = str.substr(0, str.length - 4);
	var f = str.substr(str.length - 4, 4);

	var crc16 = CRC.ToModbusCRC16(d);
	if (f == crc16) {
		return true;
	} else {
		return false;
	}
}

//解包
function solutionData(str, len) {
	str = str.toUpperCase();
	var packages = {};
	packages.top = str.substr(0, 2);
	packages.command = str.substr(2, 2);
	if (packages.command == '03') {
		var length = len * 2;
		packages.length = parseInt('0x' + str.substr(4, 2));
		if (packages.length != length) {
			return undefined;
		}
		var data = str.substr(6, length * 2);
		packages.data = data;
	} else if (packages.command == '06') {
		packages.start = str.substr(4, 4);
		packages.data = str.substr(8, 4);
	}
	packages.crc16 = str.substr(str.length - 4, 4);
	return packages;

	// var j = -1;
	// var k = 0;


	// length = length * 2;
	// for (var i = 0; i < appData.length; i++) {
	// 	if (k < length) {
	// 		var item = appData[i];
	// 		var name = item.name;
	// 		if (name == start) {
	// 			j = i;
	// 			k = 4;
	// 			var v = data.substr(0, 4);
	// 			algorithm(name, v, i);
	// 		} else if (j > -1) {
	// 			var v = data.substr(k, 4);
	// 			k = k + 4;
	// 			algorithm(name, v, i);
	// 		}
	// 	}
	// }

	//console.log(packages);
}

//16进制转明文
function algorithm(name, v1) {
	name = name.toUpperCase();
	var value = 0;
	//console.log(name, v1);
	switch (name) {
		case '1003':
		case '1011':
		case '1012':
			var v = parseInt("0x" + v1);
			value = parseFloat((v / 100).toFixed(1));
			break;
		case '1004':
			//console.log(v1, '1004');
			var v = parseInt("0x" + v1);
			value = parseFloat((v / 10 - 1000).toFixed(1));
			break;
		case '1005':
		case '1006':
		case '1007':
		case '1008':
		case '1009':
		case '100A':
		case '100B':
		case '100C':
			var v = parseInt("0x" + v1);
			value = parseFloat((v / 10 - 40).toFixed(1));
			break;
		case '1014':
		case '1016':
			var v = parseInt("0x" + v1).toString(2);
			var vArray = v.split('').reverse();
			var result = [];
			for (var i = 0; i < 16; i++) {
				var item = vArray[i];
				if (item) {
					result.push(item);
				} else {
					result.push(0);
				}
			}
			return result;
			break;
		case '100F':
			//console.log(v1);
			var h = parseInt('0x' + v1.substr(0, 2));
			var l = parseInt('0x' + v1.substr(2, 4));
			//console.log(name, h, l);
			var result = [h, l];
			return result;
			break;

			//保护参数寄存器
		case '2300': //电压V算法
		case '2302':
		case '2308':
		case '230A':
			var v = parseInt("0x" + v1);
			value = parseFloat((v / 100).toFixed(2));
			break;
		case '2310': //温度算法
		case '2312':
		case '2314':
		case '2316':
		case '2318':
		case '231A':
		case '231C':
		case '231E':
			var v = parseInt("0x" + v1);
			value = parseFloat((v / 10 - 40).toFixed(1));
			break;
		case '2320': //电流A算法
		case '2322':
		case '2324':
		case '2326':
		case '2328':
		case '232A':
		case '232C':
			var v = parseInt("0x" + v1);
			value = parseFloat((v / 10).toFixed(1));
			break;

			//硬件参数配置
		case '210A':
			var v = parseInt("0x" + v1);
			value = parseFloat((v / 100).toFixed(2));
			break;
		
		case '2110':
			var v = parseInt("0x" + v1);
			value = parseFloat((v / 10).toFixed(1));
			break;
	
		case '2113':
			var v = parseInt("0x" + v1);
			value = parseFloat((v / 10).toFixed(1));
			break;

		case '2114':
			var v = parseInt("0x" + v1);
			value = parseFloat((v / 1000).toFixed(3));
			break;

			//校准参数配置
		case '2411':
			var v = parseInt("0x" + v1);
			value = v - 1000;
			break;
		case '2420':
			var v = parseInt("0x" + v1);
			value = parseFloat((v / 100).toFixed(2));
			break;
		case '2427':
			var v = parseInt("0x" + v1);
			value = parseFloat((v / 10).toFixed(1));
			break;

		default:
			var v = parseInt("0x" + v1);
			value = v;
			break;
	}

	return value;
	//getApp().globalData.data[i].value = value;

	//console.log(getApp().globalData.data[i].value)
}

//明文转16进制
function encryption(name, v1) {
	name = name.toUpperCase();
	switch (name) {
		//保护参数寄存器
		case '2300': //电压V算法
		case '2302':
		case '2308':
		case '230A':
			v1 = v1 * 100;
			break;
		case '2310': //温度算法
		case '2312':
		case '2314':
		case '2316':
		case '2318':
		case '231A':
		case '231C':
		case '231E':
			v1 = v1 * 10 + 400;
			break;
		case '2320': //电流A算法
		case '2322':
		case '2324':
		case '2326':
		case '2328':
		case '232A':
		case '232C':
			v1 = v1 * 10;
			break;

			//硬件参数配置
		case '210A':
			v1 = v1 * 100;
			break;
		
		case '2110':
			v1 = v1 * 10;
			break;

		case '2113':
			v1 = v1 * 10;
			break;

		case '2114':
			v1 = v1 * 1000;
			break;

			//校准参数配置
		case '2411':
			v1 = v1 + 1000;
			break;
		case '2420':
			v1 = v1 * 100;
			break;
		case '2427':
			v1 = v1 * 10;
			break;
		default:

			break;
	}
	var v2 = v1.toString(16);
	if (v2 == '') {
		return '';
	}
	if (v2.indexOf('-') >= 0) {
		return '';
	}
	if (v2.length == 1) {
		return '000' + v2;
	} else if (v2.length == 2) {
		return '00' + v2;
	} else if (v2.length == 3) {
		return '0' + v2;
	} else if (v2.length == 4) {
		return v2;
	} else {
		return '';
	}
}

export default {
	readCommand: readCommand,
	bufferToHex: ab2hex,
	checkCRC16: checkCRC16,
	solutionData: solutionData,
	algorithm: algorithm,
	encryption: encryption,
	writeCommand: writeCommand
}
